* Example 3
* From Guimaraes & Portugal (2009)
* Estimates a linear regression with two fixed effects
* Replicates the estimates in Cornelissen 2008, Stata J 8: 170-189.
* felsdvsimul is the ancillary dataset in Cornelissen 2008.
* Uses a faster approach

set more off
log using example3.log,replace

use felsdvsimul, clear
gen double temp=0
gen double fe2=0
gen double lastfe2=0
foreach var of varlist y x1 x2 {
quietly {
local i=0
local dif=1
capture drop mean
egen double mean=mean(`var'), by(i)
replace `var'=`var'-mean
while abs(`dif')>epsfloat() {
replace lastfe2=fe2, nopromote
capture drop mean
egen double mean=mean(fe2), by(i)
capture drop fe2
egen double fe2=mean(`var'+mean), by(j)
replace temp=sum(reldif(fe2,lastfe2)), nopromote
local dif=temp[_N]/_N
local i=`i'+1
}
noisily di "Total Number of Iterations for `var' --> " `i'
gen double `var'_res=`var'-fe2+mean
}
}
regress y_res x1_res x2_res, nocons

* Obtaining correct standard errors
local s2=`e(rss)'/69
quietly sum x1
local N=r(N)
local varx1=r(Var)
local TSSx1=`varx1'*(`N'-1)
quietly regress x1_res x2_res
local RSSx1=e(rss)
local R2x1=1-`RSSx1'/`TSSx1'
local sebx1=sqrt(`s2'/((`N'-1)*`varx1'*(1-`R2x1')))
di "The standard error associated with x1 is --> `sebx1'"

* Obtaining the fixed effects
qui regress y_res x1_res x2_res, nocons
gen double dum=y-_b[x1_res]*x1-_b[x2_res]*x2
capture drop mean
egen double mean=mean(dum), by(i)
qui replace dum=dum-mean
local i=0
local dif=1
while abs(`dif')>epsfloat() {
qui replace lastfe2=fe2, nopromote
capture drop mean
egen double mean=mean(fe2), by(i)
capture drop fe2
egen double fe2=mean(dum+mean), by(j)
qui replace temp=sum(reldif(fe2,lastfe2)), nopromote
local dif=temp[_N]/_N
local i=`i'+1
}
di "Total Number of Iterations for fe2 --> " `i'
quietly replace dum=dum-fe2
egen double fe1=mean(dum), by(i)
regress y x1 x2 fe1 fe2, nocons

log close
exit

