
set more off
set matsize 200
*log using exper.log,replace

** Example 1
sjlog using input1a, replace
webuse nlswork
generate age2 = age^2
generate ttl_exp2 = ttl_exp^2
generate tenure2 = tenure^2
generate double fe1=0
local rss1=0
local dif=1
local i=0
while abs(`dif')>epsdouble() {
quietly {
regress ln_w age* ttl_exp* tenure* not_smsa south fe1
local rss2=`rss1'
local rss1=e(rss)
local dif=`rss2'-`rss1'
capture drop yhat
predict double yhat 
generate double temp1=ln_w-yhat+_b[fe1]*fe1
capture drop fe1
egen double fe1=mean(temp1), by(idcode)
capture drop temp1
local i=`i'+1
}
}
di "Total Number of Iterations --> " `i'
regress ln_w age* ttl_exp* tenure* not_smsa south fe1
sjlog close, replace

sjlog using input1b, replace
xtset idcode
xtreg ln_w age* ttl_exp* tenure* not_smsa south, fe
sjlog close, replace

* Example2
sjlog using input2, replace
use felsdvsimul, clear
generate double temp=0
generate double fe1=0
generate double fe2=0
local rss1=0
local dif=1
local i=0
while abs(`dif')>epsdouble() {
quietly {
regress y x1 x2 fe1 fe2
local rss2=`rss1'
local rss1=e(rss)
local dif=`rss2'-`rss1'
capture drop res
predict double res, res
replace temp=res+_b[fe1]*fe1, nopromote
capture drop fe1
egen double fe1=mean(temp), by(i)
replace temp=res+_b[fe2]*fe2, nopromote
capture drop fe2
egen double fe2=mean(temp), by(j)
local i=`i'+1
}
}
di "Total Number of Iterations --> " `i'
regress y x1 x2 fe1 fe2
sjlog close, replace

* Example3

sjlog using input3a, replace
use felsdvsimul, clear
gen double temp=0
gen double fe2=0
gen double lastfe2=0
foreach var of varlist y x1 x2 {
quietly {
local i=0
local dif=1
capture drop mean
egen double mean=mean(`var'), by(i)
replace `var'=`var'-mean
while abs(`dif')>epsfloat() {
replace lastfe2=fe2, nopromote
capture drop mean
egen double mean=mean(fe2), by(i)
capture drop fe2
egen double fe2=mean(`var'+mean), by(j)
replace temp=sum(reldif(fe2,lastfe2)), nopromote
local dif=temp[_N]/_N
local i=`i'+1
}
noisily di "Total Number of Iterations for `var' --> " `i'
gen double `var'_res=`var'-fe2+mean
}
}
regress y_res x1_res x2_res, nocons
sjlog close, replace

sjlog using input3b, replace
local s2=`e(rss)'/69
quietly sum x1
local N=r(N)
local varx1=r(Var)
local TSSx1=`varx1'*(`N'-1)
quietly regress x1_res x2_res
local RSSx1=e(rss)
local R2x1=1-`RSSx1'/`TSSx1'
local sebx1=sqrt(`s2'/((`N'-1)*`varx1'*(1-`R2x1')))
di "The standard error associated with x1 is --> `sebx1'"
sjlog close, replace


sjlog using input3c, replace
qui regress y_res x1_res x2_res, nocons
gen double dum=y-_b[x1_res]*x1-_b[x2_res]*x2
capture drop mean
egen double mean=mean(dum), by(i)
qui replace dum=dum-mean
local i=0
local dif=1
while abs(`dif')>epsfloat() {
qui replace lastfe2=fe2, nopromote
capture drop mean
egen double mean=mean(fe2), by(i)
capture drop fe2
egen double fe2=mean(dum+mean), by(j)
qui replace temp=sum(reldif(fe2,lastfe2)), nopromote
local dif=temp[_N]/_N
local i=`i'+1
}
di "Total Number of Iterations for fe2 --> " `i'
quietly replace dum=dum-fe2
egen double fe1=mean(dum), by(i)
regress y x1 x2 fe1 fe2, nocons
sjlog close, replace

****************************
sjlog using input4, replace
webuse ships, clear
qui poisson acc op_75_79 co_65_69 co_70_74 co_75_79, exposure(service) nolog
keep if e(sample)
bys ship: egen sumy=total(acc)
gen double off=0
gen double temp=0
local dif=1
local ll1=0
local i=0
while abs(`dif')>epsdouble() {
qui poisson acc op_75_79 co_65_69 co_70_74 co_75_79, off(off) nocon
local ll2=`ll1'
local ll1=e(ll)
capture drop xb
predict double xb, xb
qui replace temp=xb-off+log(service), nopromote
capture drop sumx
bys ship: egen double sumx=total(exp(temp))
qui replace off=log(sumy/sumx)+log(service), nopromote
local dif=`ll2'-`ll1'
local i=`i'+1
}
di "Total Number of Iterations --> " `i'
poisson acc op_75_79 co_65_69 co_70_74 co_75_79, nocon off(off) irr nolog
sjlog close, replace

*Example 5
sjlog using input5, replace
webuse ships, clear
local dif=1
xi: poisson acc op_75_79 i.yr_con i.ship, nolog
keep if e(sample)
bys ship: egen sumy1=total(acc)
bys yr_con: egen sumy2=total(acc)
gen double off=0
gen double temp=0
gen double temp1=0
gen double temp2=0
gen double fe1=0
gen double fe2=0
local ll1=0
local i=0
while abs(`dif')>epsdouble() {
qui poisson acc op_75_79, nocons offset(off) nolog
local ll2=`ll1'
local ll1=e(ll)
capture drop xb
predict double xb, xb
qui replace temp1=xb-off+fe2, nopromote
capture drop sumx
bys ship: egen double sumx=total(exp(temp1))
qui replace fe1=log(sumy1/sumx), nopromote
qui replace temp2=xb-off+fe1, nopromote
capture drop sumx
bys yr_con: egen double sumx=total(exp(temp2))
qui replace fe2=log(sumy2/sumx), nopromote
qui replace off=fe1+fe2
local dif=`ll2'-`ll1'
local i=`i'+1
}
di "Total Number of Iterations --> " `i'
poisson acc op_75_79, nocons offset(off) nolog
sjlog close, replace

xi: poisson acc op_75_79 i.yr_con i.ship

*Example 6
sjlog using input6, replace
webuse ships, clear
xi: nbreg acc op_75_79 co_65_69 co_70_74 co_75_79 i.ship, nolog
keep if e(sample)
egen id=group(ship)
qui sum id
local maxg=r(max)
local dif=1
local ll1=0
local i=0
gen double off1=0
gen double off2=0
while abs(`dif')>epsdouble() {
qui nbreg acc op_75_79 co_65_69 co_70_74 co_75_79, nocons offset(off1) nolog
local lna=log(e(alpha))
constraint define 1 [lnalpha]_cons=`lna'
local ll2=`ll1'
local ll1=e(ll)
capture drop xb
predict double xb, xb
qui replace off2=xb-off1, nopromote
forval j=1/`maxg' {
qui nbreg acc if id==`j', offset(off2) constraint(1) nolog
qui replace off1=_b[_cons] if e(sample), nopromote
}
local dif=`ll2'-`ll1'
local i=`i'+1
}
di "Total Number of Iterations -->" `i'
nbreg acc op_75_79 co_65_69 co_70_74 co_75_79, offset(off1) nocons nolog
sjlog close, replace

exit
